#!/usr/bin/perl
# assemblefiles.pl
#                       Rick Landau 20071120
# Include some files in other files.
# Some files are always included; others are included only if their 
#  identifier matches the user's request.
#
# CLI: assemblefiles.pl input-file [phase]
# "#include" commands in the input file may be replaced with 
#  the contents of external files. 
# Command "#include foo" always replaces command line with file foo.
# Command "#include(bar) foo" replaces command line with file foo IFF 
#  the command line specifies phase "bar".
#  If the command line specifies no phase, or a phase other than "bar", 
#  then the "#include(bar) foo" line will remain in the file for later
#  processing.  

use English; 
use newtrace;
use strict; 

defined($ARGV[1-1]) || die "Usage: fixup  mib-file [phase-id]\n";

# Process the MIB file
my $sMibFile = $ARGV[1-1];
newtrace::trace( 1,"file1|$sMibFile|");
open( MIB, '<', $sMibFile ) || die "Error: cannot open input file $sMibFile: $!";

my $sRequestedPhase = "";
$sRequestedPhase = $ARGV[2-1] if ( defined($ARGV[2-1]) );

while (<MIB>)
{
    my $sLine = $_;
    my ($sCommand,$sFilespec) = $sLine =~ 
            /^(#include|#include\([A-Za-z0-9]+\))\s+(.+)$/;
    if ( $sCommand )
    {
        # The line is a #include command.  
        my ($sCommandPhase) = $sCommand =~ /#include\(([A-Za-z0-9]+)\)/;
        newtrace::trace(3,"proc command|$sCommand|phase|$sCommandPhase|");
        if ( $sCommandPhase )
        {
            # The line includes a phase spec.
            if ( $sCommandPhase eq $sRequestedPhase ) 
            {
                # The line's phase matches the requested phase; 
                # therefore remove the command and include the file.
                fnInsertFile( $sFilespec );
            } else
            {
                # The line's phase does not match the requested phase;
                # leave the command in the file for later processing.
                # This works if there is no requested phase, too.  
                print;
            }
        } else
        {
            # The line has no phase spec; always include the file.  
            fnInsertFile( $sFilespec );
        }
    } else
    {
        # The line is not a #include command.  Emit it.  
        print;
    }
}

# Function to insert a file here.
sub fnInsertFile
{
    my ($sFilespec) = @_;
    open( hINCLUDEME, "<$sFilespec" ) || die "Error: cannot find file $sFilespec\n";
    while (<hINCLUDEME>)
    {
        print;
    }
}

#end
